/** Print the cutsize and the imbalance of a graph partition
 * @param N the number of vertices in the graph
 * @param adjBeg beginning pointer of the adjacency list of each node
 * @param adj the list of adjacent vertex indices
 * @param part the array indicating the part index of each vertex (0 or 1)
 */
void printCutSizeImbal(
    int N,
    int *adjBeg,
    int *adj,
    int *part);

/** Computing the cutsize of a graph partition
 * @param N the number of vertices in the graph
 * @param adjBeg beginning pointer of the adjacency list of each node
 * @param adj the list of adjacent vertex indices
 * @param part the array indicating the part index of each vertex (0 or 1)
 * @param maxIters the number of iterations for which the algorithm must run
 * @param epsilon the load imbalance parameter >= 1.0
 * @param algoName the name of the algorithm to be executed
 */
void bisectGraph(
    int N,
    int *adjBeg,
    int *adj,
    int *part,
    int maxIters,
    double epsilon,
    char *algoName)
{
  int procRank, numProcs;
  printCutSizeImbal(N, adjBeg, adj, part);
  MPI_Comm_rank(MPI_COMM_WORLD, &procRank);
  MPI_Comm_size(MPI_COMM_WORLD, &numProcs);

  if (strcmp(algoName, "bisect-seq") == 0) { // Sequential graph bisection
    if (procRank == 0) {
      if (numProcs != 1) {
        printf("ERROR: Sequential algorithm is ran with %d MPI processes.\n", numProcs);
        MPI_Abort(MPI_COMM_WORLD, 1);
      }
    }
    // BEGIN IMPLEMENTATION HERE

  } else if (strcmp(algoName, "bisect-a2a") == 0) { // Graph bisection with all-to-all communication
    // BEGIN IMPLEMENTATION HERE

  } else if (strcmp(algoName, "bisect-p2p") == 0) { // Graph bisection with point-to-point communication
    // BEGIN IMPLEMENTATION HERE

  } else { // Invalid algorithm name
    if (procRank == 0) {
      printf("ERROR: Invalid algorithm name: %s.\n", algoName);
      MPI_Abort(MPI_COMM_WORLD, 1);
    }
  }
}
